

#############################################################################
# Copyright (C) Labomedia May 2012
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of the GNU General Public License
#  as published by the Free Software Foundation; either version 2
#  of the License, or (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software Foundation,
#  Inc., 51 Franproplin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#
#############################################################################
#
## game.py
#
#  This script run every frame and drive the game.
#
#############################################################################

import GameLogic as gl

# List of list
players_data = gl.players_data
# My position in list
where = gl.where
# Get some variable in GameLogic
level = gl.level
ball = gl.ball
bat = gl.bat
goal = gl.goal
level_change = gl.level_change
name = gl.name
my_name = gl.my_name
tempo = gl.tempo # 2 second tempo to display final score
block = gl.block # to add only once the Rank scene
sendosc = gl.sendosc # obj to send message in OSC scene

# Debug
debug = gl.debug

# Get scenes
scenes = gl.getSceneList()


def main():
    global truc
    # Display in Game
    display(scenes, gl.getosc)
    # Info in terminal
    print_some(name, scenes, where)
    # If reset
    ball_in_center(gl.sendosc, ball)
    # Only level 1
    automatic_bat(level, ball, bat)

    # All level
    game(level, where, ball, bat, goal, players_data, block)

    # Add Rank Scene
    add_rank(goal, scenes, level, name, players_data, my_name, tempo, block, sendosc)

    # Remove Rank 2 second after the last player win
    remove_rank(goal, scenes, level, name, players_data, my_name, tempo, block, sendosc)

def add_rank(goal, scenes, level, name, players_data, my_name, tempo, block, sendosc):
    ## Don't add if Rank exist, only if block = 0
    # Add Rank if I have finished
    if block == 0:
        for scn in scenes:
            if "_players" in scn.name:
                if goal[where]["score"] == 0:
                    # Overlay Rank
                    gl.addScene("Rank")
                    block = 1
                    # Store in gl
                    gl.block = block
                    # Block my bat
                    bat[where]["activ"] = 0

def remove_rank(goal, scenes, level, name, players_data, my_name, tempo, block, sendosc):
    if block == 1:
        if tempo == 60:
            for scn in scenes :
                if "_players" in scn.name:
                    print("_players removed")
                    scn.end()

    # Remove Rank 2 second after the last player win
    # All players are finished if no value to 0 in name
    for scn in scenes :
        if "Rank" in scn.name:
            a = 0
            for key, value in name.items():
                if value != 0:
                    a += 1

            if level == 1: c = 2
            else: c = level

            if a == c :
                tempo += 1
                gl.tempo = tempo

            if tempo == 120:
                # Remove Rank
                scn.end()
                tempo += 1
                gl.tempo = tempo

    if tempo == 121:
        # New game with 1_players
        new_game(goal, scenes, level, name, players_data, my_name, tempo, block, sendosc)
        tempo = 0
        gl.tempo = tempo

def new_game(goal, scenes, level, name, players_data, my_name, tempo, block, sendosc):
    ## Remove x_players scene, add 1_players
    ## Reset some variable maybe
    for scn in scenes :
        if "_players" in scn.name:
            # Remove _players scene
            scn.end()

        # Overlay 1_player scene
        gl.addScene("1_players")

        block = 0
        gl.block = block
        gl.name = {}

def ball_in_center(sendosc, ball):
    # All level : Recenter ball if Reset
    if sendosc["reset"] == True:
        ball.localPosition = [0, 0, 1]

def print_some(n1=0, n2=0, n3=0, n4=0, n5=0):
    # Some info in terminal to debug
    if gl.tp == 30:
        print(n1, n2, n3, n4, n5)

def automatic_bat(level, ball, bat):
    # Only level 1 Set automatic bat with one player
    for scn in scenes :
        if scn.name == "1_players":
            y = 0.2 * ball.localPosition[1]
            bat[1].localPosition = [9.5, y, 1]

def display(scenes, getosc):
    #Display some message in Game
    for scn in scenes :
        if scn.name == "OSC": # to be sure that getosc exist
            if level == 0 :
                getosc["Text"] = "Problem with Network or Server"
            else:
                getosc["Text"] = "Player number =  " + str(level) + "\n" + "Help = H"

def game(level, where, ball, bat, goal, players_data, block):
    ### Set ball, bat, goal

    ## Score must always be positive or null
    for scn in scenes :
        if "_players" in scn.name : # to be sure that the good scene exist
            # Get prop score in goal
            if level == 1: b = 2 # Also in goal11: level = 1 but 2 players
            else: b = level
            for g in range(b):
                if goal[g]["score"] <= 0:
                    goal[g]["score"] = 0

    ## Set ball position if I am not master
    # The first player in list is always the master
    if where != 0 :
        # Disable ball rigid body with prop master=False in ball
        ball["master"] = False
        ball.localPosition = [players_data[0][2], players_data[0][3], 1]

    ## Set score with the good score from players
    # Normal game
    if debug == 0:
        # For all players
        for player in range(level):
            # But not for me
            if player != where:
                goal[player]["score"] = players_data[player][8]


    ## Set prop activ in bat : bat move with keyboard only if activ = 1
    # The good bat is my position in players_data: where
    for scn in scenes :
        if "_players" in scn.name :
            # Bat doesn't move if I lost
            if block == 0:
                # Disable my bat
                bat[where]["activ"] = 1

    ## Set all other bat position
    # Normal game
    if debug == 0:
        # For all players
        for player in range(level):
            # But not for me
            if player != where:
                print(players_data[player][5], players_data[player][6])
                bat[player].localPosition = [players_data[player][5], players_data[player][6], 1]

    ## Movement for fictive players to debug
    if debug == 1:
        for other in range(level):
            if other != where:
                if gl.tp < 60 :
                    a = 0.05
                if gl.tp >= 60 :
                    a = -0.05
                # Set amount to move
                movement = [ 0.0, a, 0.0]
                # Use local axis
                local = True
                # Move game object
                try:
                    bat[other].applyMovement(movement, local)
                except: pass


main()
